/*
 ******************************* Creazione file *******************************
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago             	Data di creazione : 23/12/98
 * Codice   :
 *
 * Declarations of Critical Section and Syncronization Objects
 *
 ******************************* Revisione file *******************************
 *-------------------------------------|--------------------------------------*
 * Revisore : 								Data di revisione :
 * Versione :
 *
 * Note:
 *
 */


#ifndef	__CRITICAL_SECTION_H
#define	__CRITICAL_SECTION_H

/******************************** Definizioni *********************************
*--------------------------------------|--------------------------------------*/

/*************************** Dichiarazione dei tipi ***************************
*--------------------------------------|--------------------------------------*/
/********************************** Inlines ***********************************
*--------------------------------------|--------------------------------------*/

class TCriticalSection
{
  public:
    TCriticalSection();
   ~TCriticalSection();

    class TLock 
	{
      public:
        TLock(const TCriticalSection&);
       ~TLock();
      private:
        const TCriticalSection& CritSecObj;
    };
    friend TLock;

private:
    CRITICAL_SECTION CritSec;

    TCriticalSection(const TCriticalSection&);
    const TCriticalSection& operator =(const TCriticalSection&);
};

/********************************** Inlines ***********************************
*--------------------------------------|--------------------------------------*/

//----------------------------------------------------------------------------
// TCriticalSection 
//
//

inline TCriticalSection::TCriticalSection()
{
  ::InitializeCriticalSection(const_cast<CRITICAL_SECTION*>(&CritSec));
}

//
// Use system call to destroy the CRITICAL_SECTION object.
//
inline TCriticalSection::~TCriticalSection()
{
  ::DeleteCriticalSection(const_cast<CRITICAL_SECTION*>(&CritSec));
}

//
// Use system call to lock the CRITICAL_SECTION object.
//
inline TCriticalSection::TLock::TLock(const TCriticalSection& sec)
:
  CritSecObj(sec)
{
  ::EnterCriticalSection(const_cast<CRITICAL_SECTION*>(&CritSecObj.CritSec));
}

//
// Use system call to unlock the CRITICAL_SECTION object.
//
inline TCriticalSection::TLock::~TLock()
{
  ::LeaveCriticalSection(const_cast<CRITICAL_SECTION*>(&CritSecObj.CritSec));
}


class TSync
{
protected:
    TSync();
    TSync(const TSync&);
    const TSync& operator =(const TSync&);

    class TLock : private TCriticalSection::TLock
    {
      public:
        TLock(const TSync*);
      private:
        static const TCriticalSection& GetRef(const TSync*);
    };
    friend TLock;

private:
    TCriticalSection CritSec;
};

/********************************** Inlines ***********************************
*--------------------------------------|--------------------------------------*/

//
// Copy constructor does not copy the TCriticalSection object,
// since the new object is not being used in any of its own
// member functions. This means that the new object must start
// in an unlocked state.
//
inline TSync::TSync()
{
}

//
inline TSync::TSync(const TSync&)
{
}

//
// Does not copy the TCriticalSection object, since the new  object is not
// being used in any of its own member functions.
// This means that the new object must start in an unlocked state.
//
inline const TSync& TSync::operator =(const TSync&)
{
  return *this;
}

//
// Locks the TCriticalSection object in the TSync object.
//
inline TSync::TLock::TLock(const TSync* sync)
:
  TCriticalSection::TLock(GetRef(sync))
{
}

  //
// Returns a reference to the TCriticalSection object contained in the TSync
// object.
//
// In the diagnostic version, checks for a null pointer.
//
inline const TCriticalSection& TSync::TLock::GetRef(const TSync* sync)
{
	return sync->CritSec;
}

/************************* Variabili globali esterne **************************
*--------------------------------------|--------------------------------------*/

/************************** Prototipi delle funzioni **************************
*--------------------------------------|--------------------------------------*/

/*********************************** Macro ************************************
*--------------------------------------|--------------------------------------*/

#endif	//	#define	__CRITICAL_SECTION_H
